using System;
using System.Collections.ObjectModel;
using System.IO;
using System.Reflection;
using ThinkGeo.MapSuite.Drawing;
using ThinkGeo.MapSuite.Portable;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;

namespace ThinkGeo.MapSuite.Layers
{
    [Serializable]
    public class LegendAdornmentLayer : AdornmentLayer
    {
        [Obfuscation(Exclude = true)]
        private GeoCollection<LegendItem> legendItems;

        [Obfuscation(Exclude = true)]
        private float titleHeight;

        [Obfuscation(Exclude = true)]
        private float footerHeight;

        [Obfuscation(Exclude = true)]
        private float rowMaxWidth;

        [Obfuscation(Exclude = true)]
        private Collection<LegendDrawingParameters> offsets;

        [Obfuscation(Exclude = true)]
        private LegendItem title;

        [Obfuscation(Exclude = true)]
        private LegendItem footer;

        [Obfuscation(Exclude = true)]
        private LegendContentResizeMode contentResizeMode;

        public LegendAdornmentLayer()
        {
            legendItems = new GeoCollection<LegendItem>();
            Width = 160;
            Height = 200;
            BackgroundMask = AreaStyles.CreateLinearGradientStyle(new GeoColor(255, 255, 255, 255), new GeoColor(255, 230, 230, 230), 90, GeoColor.SimpleColors.Black);
            Location = AdornmentLocation.LowerLeft;
            contentResizeMode = LegendContentResizeMode.Default;
        }

        public LegendItem Title
        {
            get { return title; }
            set { title = value; }
        }

        public LegendItem Footer
        {
            get { return footer; }
            set { footer = value; }
        }

        public GeoCollection<LegendItem> LegendItems
        {
            get { return legendItems; }
        }

        public LegendContentResizeMode ContentResizeMode
        {
            get { return contentResizeMode; }
            set { contentResizeMode = value; }
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            GeoImage bitmap = null;
            MemoryStream memoryStream = null;
            try
            {
                bitmap = new GeoImage((int)Width, (int)Height);

                GeoCanvas adornmentGeoCanvas = GeoCanvas.CreatePlatformGeoCanvas();

                adornmentGeoCanvas.BeginDrawing(bitmap, canvas.CurrentWorldExtent, canvas.MapUnit);
                if (BackgroundMask != null)
                {
                    // Here -2 is for displaying all the 4 edges of the rectangle, otherwise the lower one and the right one cannot be displayed properly since the rectangle is too big for the canvas.
                    DrawingRectangleF backgroudExtent = new DrawingRectangleF(adornmentGeoCanvas.Width / 2, adornmentGeoCanvas.Height / 2, adornmentGeoCanvas.Width - 2, adornmentGeoCanvas.Height - 2);
                    BackgroundMask.DrawSample(adornmentGeoCanvas, backgroudExtent);
                }

                float widthMagnificationRatio = adornmentGeoCanvas.Width / 160;
                float heightMagnificationRatio = adornmentGeoCanvas.Height / 120;

                if (ContentResizeMode == LegendContentResizeMode.Fixed || ContentResizeMode == LegendContentResizeMode.Default)
                {
                    widthMagnificationRatio = 1;
                    heightMagnificationRatio = 1;
                }

                if (title != null)
                {
                    title.Ratio = widthMagnificationRatio;
                }
                if (footer != null)
                {
                    footer.Ratio = widthMagnificationRatio;
                }

                foreach (var item in legendItems)
                {
                    item.Ratio = widthMagnificationRatio;
                }

                CountTheOffsetOfEachLegendItem(widthMagnificationRatio, heightMagnificationRatio);

                if (Title != null)
                {
                    DrawTitle(labelsInAllLayers, adornmentGeoCanvas);
                }

                int countPostion = 0;
                foreach (LegendItem legendItem in LegendItems)
                {
                    DrawEachLegendItem(labelsInAllLayers, adornmentGeoCanvas, countPostion, legendItem);
                    countPostion++;
                }

                if (Footer != null)
                {
                    DrawFooter(labelsInAllLayers, adornmentGeoCanvas);
                }

                adornmentGeoCanvas.EndDrawing();

                ScreenPointF drawingLocation = GetDrawingLocation(canvas, (int)Width, (int)Height);
                GeoImage geoImage = new GeoImage(PclSystem.Current.Resolve<INativeImage>().Clone(bitmap.NativeImage));
                canvas.DrawScreenImageWithoutScaling(geoImage, (drawingLocation.X + Width / 2), (drawingLocation.Y + Height / 2), DrawingLevel.LevelOne, 0, 0, 0);
                geoImage.Dispose();
            }
            finally
            {
                if (memoryStream != null) memoryStream.Dispose();
                if (bitmap != null) bitmap.Dispose();
            }
        }

        private void DrawEachLegendItem(Collection<SimpleCandidate> labelsInAllLayers, GeoCanvas adornmentGeoCanvas, int countPostion, LegendItem legendItem)
        {
            if (!IsEnoughPlaceToDraw(legendItem, "LegendItem"))
            {
                return;
            }

            //float xOffset = offsets[countPostion].XOffset;
            //float yOffset = offsets[countPostion].YOffset;

            legendItem.Draw(adornmentGeoCanvas, labelsInAllLayers, offsets[countPostion]);
        }

        private void DrawTitle(System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers, GeoCanvas adornmentGeoCanvas)
        {
            if (!IsEnoughPlaceToDraw(Title, "Title"))
            {
                return;
            }

            LegendDrawingParameters offsets = new LegendDrawingParameters();
            offsets.XOffset = 0;
            offsets.YOffset = 0;

            Title.Draw(adornmentGeoCanvas, labelsInAllLayers, offsets);
        }

        private void DrawFooter(System.Collections.ObjectModel.Collection<SimpleCandidate> labelsInAllLayers, GeoCanvas adornmentGeoCanvas)
        {
            if (!IsEnoughPlaceToDraw(Footer, "Footer"))
            {
                return;
            }

            LegendDrawingParameters offsets = new LegendDrawingParameters();
            offsets.XOffset = 0;
            offsets.YOffset = (adornmentGeoCanvas.Height - Footer.BottomPadding - footerHeight - Footer.TopPadding);

            Footer.Draw(adornmentGeoCanvas, labelsInAllLayers, offsets);
        }

        private void CountTheOffsetOfEachLegendItem(float widthEnlargedRatio, float heightEnlargedRatio)
        {
            offsets = new Collection<LegendDrawingParameters>();
            LegendDrawingParameters currentOffsets = new LegendDrawingParameters();
            currentOffsets.XOffset = 0;
            currentOffsets.YOffset = 0;

            float titlePadding = 0;
            float footerPadding = 0;
            if (Title != null)
            {
                titleHeight = Title.Height;
                titlePadding = Title.TopPadding + Title.BottomPadding;
            }
            if (Footer != null)
            {
                footerHeight = Footer.Height;
                footerPadding = Footer.TopPadding + Footer.BottomPadding;
            }

            bool firstLegendItem = true;
            float titleFooterHeight = titleHeight + footerHeight + titlePadding + footerPadding;
            float allItemsHeight = 0;
            float lastLegendItemHeight = 0;
            float lastLegendItemTopPadding = 0;
            //float ratio = Width / 192;
            foreach (LegendItem legendItem in LegendItems)
            {
                LegendDrawingParameters tempOffsets = new LegendDrawingParameters();
                if (titleFooterHeight + (allItemsHeight + legendItem.Height + legendItem.TopPadding + legendItem.BottomPadding) * widthEnlargedRatio <= Height * heightEnlargedRatio)//LegendItems' real Height(not LegenItem.Height) is decided by the widthEnlargedRatio of the legend
                {
                    if (!firstLegendItem)
                    {
                        currentOffsets.YOffset = currentOffsets.YOffset + lastLegendItemHeight + lastLegendItemTopPadding;
                    }
                    else
                    {
                        currentOffsets.YOffset = 0 + titleHeight + titlePadding;
                    }
                    allItemsHeight = allItemsHeight + legendItem.Height + legendItem.TopPadding;
                    if (legendItem.Width + legendItem.LeftPadding > rowMaxWidth)
                    {
                        rowMaxWidth = legendItem.Width + legendItem.LeftPadding;
                    }
                }
                else
                {
                    currentOffsets.XOffset = currentOffsets.XOffset + rowMaxWidth + legendItem.LeftPadding;
                    currentOffsets.YOffset = 0 + titleHeight + titlePadding;
                    allItemsHeight = legendItem.Height + legendItem.TopPadding + titleHeight + footerHeight + titlePadding + footerPadding;
                    rowMaxWidth = legendItem.Width + legendItem.LeftPadding;
                }

                tempOffsets.XOffset = currentOffsets.XOffset;
                tempOffsets.YOffset = currentOffsets.YOffset;

                if (!offsets.Contains(tempOffsets))
                {
                    offsets.Add(tempOffsets);
                }

                firstLegendItem = false;
                lastLegendItemHeight = legendItem.Height + legendItem.BottomPadding;
                lastLegendItemTopPadding = legendItem.TopPadding;
            }
        }

        private bool IsEnoughPlaceToDraw(LegendItem legendItem, string type)
        {
            bool result = false;
            double extraHeight = 0;

            if (type.Equals("Footer"))
            {
                extraHeight = footerHeight + Footer.TopPadding + Footer.BottomPadding;
            }
            else if (type.Equals("LegendItem"))
            {
                double footerTotalHeight = 0;
                if (Footer != null)
                {
                    footerTotalHeight = footerHeight + Footer.TopPadding + Footer.BottomPadding;
                }
                extraHeight = footerTotalHeight + legendItem.Height + legendItem.TopPadding + legendItem.BottomPadding;
            }
            double titleTotalHeight = 0;
            if (Title != null)
            {
                titleTotalHeight = titleHeight + Title.TopPadding + Title.BottomPadding;
            }
            if (titleTotalHeight + extraHeight < Height)
            {
                result = true;
            }
            return result;
        }
    }
}